#!/usr/bin/python3
# -*- coding: utf-8 -*-

#处理包信息的dbus接口

from model.globals import Globals
import dbus
import dbus.service
import apt
import json
import subprocess

INTERFACE = "com.kylin.softwarecenter.packageInfo"
UKPATH = "/com/kylin/softwarecenter/packageInfo"
class PackageInfomation(dbus.service.Object):
    aptCache = None
    def __init__(self,bus, mainloop):
        self.bus_name = dbus.service.BusName(INTERFACE, bus=bus)
        dbus.service.Object.__init__(self, self.bus_name, UKPATH)
        self.mainloop = mainloop
        self.initAptCache()
        self.reloadAptCache()
        pass

    def initAptCache(self):
        try:
            self.aptCache = apt.Cache()
            self.aptCache.open()
        except Exception as e:
            Globals.logError.logger.error("init apt cache error : %s"  % str(e))
            self.aptCache = None

    @dbus.service.method(INTERFACE, in_signature='', out_signature='')
    def exit(self):
        self.mainloop.quit()

    @dbus.service.method(INTERFACE, in_signature='', out_signature='b')
    def reloadAptCache(self):
        if(self.aptCache != None):
            self.aptCache.close()
            self.aptCache.open()
        return True

    @dbus.service.method(INTERFACE, in_signature='s', out_signature='s')
    def getPackageInfo(self,packageName):
        msg = {}
        try:
            package = self.aptCache[packageName.lower()]
            if(package.is_installed and package.candidate.installed_size != 0):
                msg["size"] = package.candidate.installed_size
            else:
                msg["size"] = package.candidate.size
            msg["version"] = package.candidate.version
            msg["aptDescription"] = package.candidate.description
            msg["aptSummary"] = package.candidate.summary
            msg["installStatue"] = package.is_installed
            msg["updatable"] = package.is_upgradable
        except Exception as e:
            Globals.logError.logger.error("get "+packageName + " packageInfo error : %s" % str(e))
        msgJson = json.dumps(msg)
        return msgJson

    @dbus.service.method(INTERFACE, in_signature='s', out_signature='s')
    def getPackageUpdateStatus(self, packageName):
        msg = {}
        try:
            package = self.aptCache[packageName.lower()]
            msg["installStatue"] = package.is_installed
            msg["updatable"] = package.is_upgradable
        except Exception as e:
            Globals.logError.logger.error("get "+packageName+" update status error : %s" % str(e))
        msgJson = json.dumps(msg)
        return msgJson

    @dbus.service.method(INTERFACE, in_signature='s', out_signature='s')
    def getPackageInstallStatus(self, packageName):
        msg = {}
        try:
            package = self.aptCache[packageName.lower()]
            msg["installStatue"] = package.is_installed
        except Exception as e:
            Globals.logError.logger.error("get "+packageName+" install status error : %s" % str(e))
        msgJson = json.dumps(msg)
        return msgJson


    @dbus.service.method(INTERFACE, in_signature='s', out_signature='s')
    def getPackageVersion(self, packageName):
        msg = {}
        try:
            package = self.aptCache[packageName.lower()]
            msg["version"] = package.candidate.version
        except Exception as e:
            Globals.logError.logger.error("get "+packageName+" version error : %s" % str(e))
        msgJson = json.dumps(msg)
        return msgJson

    @dbus.service.method(INTERFACE, in_signature='s', out_signature='s')
    def getPackageVersionList(self, packageName):
        msg = {}
        version = ""
        try:
            package = self.aptCache[packageName.lower()]
            try:
                # 使用dpkg命令查询包版本
                output = subprocess.check_output(['dpkg', '-l', packageName], stderr=subprocess.STDOUT)

                # 输出是多行的，我们需要找到包含版本信息的那一行
                for line in output.decode('utf-8').split('\n'):
                    if packageName in line:
                        # 版本信息通常在输出的第三列
                        version = line.split()[2]

            except subprocess.CalledProcessError as e:
                # 如果命令执行出错，打印错误信息并返回None
                Globals.logError.logger.error("get "+packageName+" version list error : %s" % str(e))
            msg["installedVersion"] = version
            msg["versionList"] = package.versions()
        except Exception as e:
            Globals.logError.logger.error("get "+packageName+" version list error : %s" % str(e))
        msgJson = json.dumps(msg)
        return msgJson
    @dbus.service.method(INTERFACE, in_signature='s', out_signature='s')
    def getPackageInstalledVersion(self, packageName):
        msg = {}
        version = ""
        try:
            # 使用dpkg命令查询包版本
            output = subprocess.check_output(['dpkg', '-l', packageName], stderr=subprocess.STDOUT)

            # 输出是多行的，我们需要找到包含版本信息的那一行
            for line in output.decode('utf-8').split('\n'):
                if packageName in line:
                    # 版本信息通常在输出的第三列
                    version = line.split()[2]

        except subprocess.CalledProcessError as e:
            # 如果命令执行出错，打印错误信息并返回None
            Globals.logError.logger.error("get "+packageName+" installed version list error : %s" % str(e))
        msg["installedVersion"] = version
        msgJson = json.dumps(msg)
        return msgJson

